
pub enum MType {
    Gauge,
    Counter,
}

impl std::fmt::Display for MType {
    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
        match self {
            MType::Gauge => write!(f, "gauge"),
            MType::Counter => write!(f, "counter")
        }
    }
}

struct MetricLabel {
    label: String,
    value: String,
}

impl MetricLabel {
    fn prometheus(&self) -> String {
        return format!("{}=\"{}\"", self.label, self.value)
    }
}

struct MetricData {
    labels: Vec<MetricLabel>,
    value: f64
}

impl MetricData {
    fn new(value: f64) -> MetricData {
        return MetricData {
            labels: Vec::new(),
            value: value,
        };
    }
    fn add_label(&mut self, label: String, value: String) {
        self.labels.push(MetricLabel{label: label, value: value})
    }
    fn rounded_val(&self) -> String {
        if self.value.fract() == 0.0 {
            return (self.value.trunc() as i64).to_string();
        }
        return self.value.to_string();
    }
}

pub struct Metric {
    name: String,
    mtype: MType,
    data: Vec<MetricData>
}

impl Metric {
    pub fn new(name: &str, mtype: MType) -> Metric {
        return Metric {
            name: String::from(name),
            mtype: mtype,
            data: Vec::new()
        };
    }
    pub fn new_val(name: &str, mtype: MType, value: f64) -> Metric {
        return Metric {
            name: String::from(name),
            mtype: mtype,
            data: vec![MetricData::new(value)]
        };
    }
}

pub struct Metrics {
    pub metrics: Vec<Metric>,
    pub processtime: f64
}

impl Metrics {
    pub fn prometheus(&self) -> String {
        let mut type_section = String::new();
        let mut metric_section = String::new();

        for m in &self.metrics {
            type_section.push_str(&format!("# TYPE {} {}\n", m.name, m.mtype));
            for d in &m.data {
                let mut labels = String::new();
                for (i, l) in d.labels.iter().enumerate() {
                    labels.push_str(&l.prometheus());
                    if i != d.labels.len()-1 {
                        labels.push(',')
                    }
                }
                if labels.is_empty() {
                    metric_section.push_str(&format!("{} {}\n", m.name, d.rounded_val()));
                } else {
                    metric_section.push_str(&format!("{}{{{}}} {}\n", m.name, labels, d.rounded_val()));
                }
            }
        }

        return type_section+&metric_section;
    }

    pub fn new() -> Metrics {
        return Metrics {
            metrics: Vec::new(),
            processtime: 0.0
        };
    }
}
